package com.cubetiqs.sql;

import javax.sql.DataSource;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.SQLFeatureNotSupportedException;
import java.util.Properties;
import java.util.logging.Logger;

public final class ConnectionFactory {
    public static class ConnectionBuilder {
        private String driver;
        private String url;
        private String user;
        private String password;

        private Properties info;

        public ConnectionBuilder driver(String driver) {
            this.driver = driver;
            return this;
        }

        public ConnectionBuilder url(String url) {
            this.url = url;
            return this;
        }

        public ConnectionBuilder user(String user) {
            this.user = user;
            return this;
        }

        public ConnectionBuilder password(String password) {
            this.password = password;
            return this;
        }

        public ConnectionBuilder info(Properties info) {
            this.info = info;
            return this;
        }

        public ConnectionBuilder properties(DataSourceProps properties) {
            if (properties != null) {
                this.driver = properties.getDriver();
                this.url = properties.getUrl();
                this.user = properties.getUser();
                this.password = properties.getPassword();
                if (properties.getProperties() != null && !properties.getProperties().isEmpty()) {
                    if (this.info == null) {
                        this.info = new Properties();
                    }
                    this.info.putAll(properties.getProperties());
                }
            }
            return this;
        }

        public Connection build() throws SQLException {
            if (driver == null) {
                return null;
            }

            try {
                Class.forName(driver);
            } catch (ClassNotFoundException e) {
                throw new SQLException(e);
            }

            if (user == null && password == null) {
                if (info != null) {
                    return DriverManager.getConnection(url, info);
                }

                return DriverManager.getConnection(url);
            }

            return DriverManager.getConnection(url, user, password);
        }
    }

    public static ConnectionBuilder builder() {
        return new ConnectionBuilder();
    }

    public static class SimpleDataSource implements DataSource {
        private final ConnectionBuilder connectionBuilder;
        private PrintWriter printWriter;
        private int loginTimeout = 0;

        public SimpleDataSource(ConnectionBuilder connectionBuilder) {
            this.connectionBuilder = connectionBuilder;
            this.printWriter = new PrintWriter(System.out);
        }

        public SimpleDataSource(ConnectionBuilder connectionBuilder, PrintWriter printWriter) {
            this.connectionBuilder = connectionBuilder;
            this.printWriter = printWriter;
        }

        @Override
        public Connection getConnection() throws SQLException {
            return connectionBuilder.build();
        }

        @Override
        public Connection getConnection(String username, String password) throws SQLException {
            return connectionBuilder.user(username).password(password).build();
        }

        @Override
        public PrintWriter getLogWriter() throws SQLException {
            return printWriter;
        }

        @Override
        public void setLogWriter(PrintWriter out) throws SQLException {
            this.printWriter = out;
        }

        @Override
        public void setLoginTimeout(int seconds) throws SQLException {
            this.loginTimeout = seconds;
        }

        @Override
        public int getLoginTimeout() throws SQLException {
            return loginTimeout;
        }

        @Override
        public Logger getParentLogger() throws SQLFeatureNotSupportedException {
            return Logger.getGlobal();
        }

        @Override
        public <T> T unwrap(Class<T> iface) throws SQLException {
            return null;
        }

        @Override
        public boolean isWrapperFor(Class<?> iface) throws SQLException {
            return false;
        }
    }

    public static DataSource createDataSource(ConnectionBuilder builder) {
        return new SimpleDataSource(builder);
    }

    public static JdbcDataFactory createDataFactory(ConnectionFactory.ConnectionBuilder builder) {
        if (builder == null) {
            return null;
        }

        return JdbcDataFactory.create(new JdbcDataQueryImpl(builder::build));
    }
}
