package com.cubetiqs.sql;

import com.cubetiqs.sql.mapper.MapperProvider;
import com.cubetiqs.sql.mapper.RowMapperProvider;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

public class JdbcDataQueryImpl implements JdbcDataQuery {
    private final IDataManager manager;

    public JdbcDataQueryImpl(IDataManager manager) {
        this.manager = manager;
    }

    @Override
    public IDataManager getManager() {
        return manager;
    }

    @Override
    public IExecuteResult<List<Object>> queryForList(String sql) {
        ExecuteResult.ExecuteResultBuilder<List<Object>> builder = ExecuteResult.builder();

        try (Statement statement = manager.getConnection().createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY)) {
            builder.data(ResultSetUtil.getDataAsList(statement.executeQuery(sql)));
        } catch (SQLException ex) {
            builder.error(ex);
        }

        return builder.build();
    }

    @Override
    public IExecuteResult<List<Object>> queryForList(String sql, Object... args) {
        ExecuteResult.ExecuteResultBuilder<List<Object>> builder = ExecuteResult.builder();

        try (PreparedStatement statement = manager.getConnection().prepareStatement(sql, ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY)) {
            ResultSetUtil.applyParameterized(statement, sql, args);
            builder.data(ResultSetUtil.getDataAsList(statement.executeQuery()));
        } catch (SQLException ex) {
            builder.error(ex);
        }

        return builder.build();
    }

    @Override
    public <R> IExecuteResult<List<R>> queryForList(String sql, RowMapperProvider<R> mapper) {
        ExecuteResult.ExecuteResultBuilder<List<R>> builder = ExecuteResult.builder();
        List<R> result = new LinkedList<>();
        try (Statement statement = manager.getConnection().createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY)) {
            ResultSet rs = statement.executeQuery(sql);
            int rowNum = 0;
            while (rs.next()) {
                rowNum++;
                result.add(mapper.map(rs, rowNum));
            }
        } catch (SQLException ex) {
            builder.error(ex);
        }

        return builder.data(result).build();
    }

    @Override
    public <R> IExecuteResult<List<R>> queryForList(String sql, RowMapperProvider<R> mapper, Object... args) {
        ExecuteResult.ExecuteResultBuilder<List<R>> builder = ExecuteResult.builder();
        List<R> result = new LinkedList<>();
        try (PreparedStatement statement = manager.getConnection().prepareStatement(sql, ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY)) {
            ResultSetUtil.applyParameterized(statement, sql, args);
            ResultSet rs = statement.executeQuery();
            int rowNum = 0;
            while (rs.next()) {
                rowNum++;
                result.add(mapper.map(rs, rowNum));
            }
        } catch (SQLException ex) {
            builder.error(ex);
        }

        return builder.data(result).build();
    }

    @Override
    public IExecuteResult<Object> queryForObject(String sql) {
        ExecuteResult.ExecuteResultBuilder<Object> builder = ExecuteResult.builder();

        try (Statement statement = manager.getConnection().createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY)) {
            builder.data(ResultSetUtil.getDataAsObject(statement.executeQuery(sql)));
        } catch (SQLException ex) {
            builder.error(ex);
        }

        return builder.build();
    }

    @Override
    public IExecuteResult<Object> queryForObject(String sql, Object... args) {
        ExecuteResult.ExecuteResultBuilder<Object> builder = ExecuteResult.builder();

        try (PreparedStatement statement = manager.getConnection().prepareStatement(sql, ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY)) {
            ResultSetUtil.applyParameterized(statement, sql, args);
            builder.data(ResultSetUtil.getDataAsObject(statement.executeQuery()));
        } catch (SQLException ex) {
            builder.error(ex);
        }

        return builder.build();
    }

    @Override
    public <R> IExecuteResult<R> query(MapperProvider<Map<String, Object>, R> mapper, String sql, Class<R> clazz) {
        ExecuteResult.ExecuteResultBuilder<R> builder = ExecuteResult.builder();

        try (Statement statement = manager.getConnection().createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY)) {
            builder.data(ResultSetUtil.getDataAsType(mapper, statement.executeQuery(sql), clazz));
        } catch (SQLException ex) {
            builder.error(ex);
        }

        return builder.build();
    }

    @Override
    public <R> IExecuteResult<R> query(MapperProvider<Map<String, Object>, R> mapper, String sql, Class<R> clazz, Object... args) {
        ExecuteResult.ExecuteResultBuilder<R> builder = ExecuteResult.builder();

        try (PreparedStatement statement = manager.getConnection().prepareStatement(sql, ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY)) {
            ResultSetUtil.applyParameterized(statement, sql, args);
            builder.data(ResultSetUtil.getDataAsType(mapper, statement.executeQuery(), clazz));
        } catch (SQLException ex) {
            builder.error(ex);
        }

        return builder.build();
    }

    @Override
    public <R> IExecuteResult<R> query(String sql, RowMapperProvider<R> mapper) {
        ExecuteResult.ExecuteResultBuilder<R> builder = ExecuteResult.builder();
        try (Statement statement = manager.getConnection().createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY)) {
            ResultSet rs = statement.executeQuery(sql);
            if (rs.next()) {
                builder.data(mapper.map(rs, 1));
            }
        } catch (SQLException ex) {
            builder.error(ex);
        }

        return builder.build();
    }

    @Override
    public <R> IExecuteResult<R> query(String sql, RowMapperProvider<R> mapper, Object... args) {
        ExecuteResult.ExecuteResultBuilder<R> builder = ExecuteResult.builder();
        try (PreparedStatement statement = manager.getConnection().prepareStatement(sql, ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY)) {
            ResultSetUtil.applyParameterized(statement, sql, args);
            ResultSet rs = statement.executeQuery();
            if (rs.next()) {
                builder.data(mapper.map(rs, 1));
            }
        } catch (SQLException ex) {
            builder.error(ex);
        }

        return builder.build();
    }

    @Override
    public long count(String sql) {
        try (Statement statement = manager.getConnection().createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY)) {
            ResultSet rs = statement.executeQuery(sql);
            if (rs.next()) {
                return rs.getLong(1);
            } else {
                return 0;
            }
        } catch (SQLException ex) {
            return 0;
        }
    }

    @Override
    public long count(String sql, Object... args) {
        try (PreparedStatement statement = manager.getConnection().prepareStatement(sql, ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY)) {
            ResultSetUtil.applyParameterized(statement, sql, args);
            ResultSet rs = statement.executeQuery();
            if (rs.next()) {
                return rs.getLong(1);
            } else {
                return 0;
            }
        } catch (SQLException ex) {
            return 0;
        }
    }
}
